/* 
/ gGraph.h
/
/ public gGraph declarations
/
/ version 1.0, 2010 July 20
/
/ Author: Sandro Furieri a.furieri@lqt.it
/
/ Copyright (C) 2010  Alessandro Furieri
/
/    This program is free software: you can redistribute it and/or modify
/    it under the terms of the GNU Lesser General Public License as published by
/    the Free Software Foundation, either version 3 of the License, or
/    (at your option) any later version.
/
/    This program is distributed in the hope that it will be useful,
/    but WITHOUT ANY WARRANTY; without even the implied warranty of
/    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/    GNU Lesser General Public License for more details.
/
/    You should have received a copy of the GNU Lesser General Public License
/    along with this program.  If not, see <http://www.gnu.org/licenses/>.
/
*/

#ifdef DLL_EXPORT
#define GGRAPH_DECLARE __declspec(dllexport)
#else
#define GGRAPH_DECLARE extern
#endif

#ifndef _GGRAPH_H
#define _GGRAPH_H

#ifdef __cplusplus
extern "C"
{
#endif

#define GGRAPH_COLORSPACE_UNKNOWN	-1000
#define GGRAPH_COLORSPACE_MONOCHROME	1001
#define GGRAPH_COLORSPACE_PALETTE	1002
#define GGRAPH_COLORSPACE_GRAYSCALE	1003
#define GGRAPH_COLORSPACE_TRUECOLOR	1004
#define GGRAPH_COLORSPACE_GRID		1005

#define GGRAPH_SAMPLE_UNKNOWN	-1500
#define GGRAPH_SAMPLE_UINT		1501
#define GGRAPH_SAMPLE_INT		1502
#define GGRAPH_SAMPLE_FLOAT		1503

#define GGRAPH_TIFF_DEFAULT	-1001

#define GGRAPH_TIFF_LAYOUT_STRIPS	2001
#define GGRAPH_TIFF_LAYOUT_TILES	2002

#define GGRAPH_TIFF_COMPRESSION_UNKNOWN		-3000
#define GGRAPH_TIFF_COMPRESSION_NONE		3001
#define GGRAPH_TIFF_COMPRESSION_CCITTFAX3	3002
#define GGRAPH_TIFF_COMPRESSION_CCITTFAX4	3003
#define GGRAPH_TIFF_COMPRESSION_LZW		3004
#define GGRAPH_TIFF_COMPRESSION_DEFLATE		3005
#define GGRAPH_TIFF_COMPRESSION_JPEG		3006

#define GGRAPH_IMAGE_UNKNOWN	-4000
#define GGRAPH_IMAGE_GIF	4001
#define GGRAPH_IMAGE_PNG	4002
#define GGRAPH_IMAGE_JPEG	4003
#define GGRAPH_IMAGE_TIFF	4004
#define GGRAPH_IMAGE_GEOTIFF	4005

#define GGRAPH_OK	0
#define GGRAPH_ERROR				-1
#define GGRAPH_INVALID_IMAGE		-2
#define GGRAPH_INSUFFICIENT_MEMORY	-3
#define GGRAPH_FILE_OPEN_ERROR		-4
#define GGRAPH_FILE_READ_ERROR		-5
#define GGRAPH_FILE_WRITE_ERROR		-6
#define GGRAPH_GIF_CODEC_ERROR		-7
#define GGRAPH_PNG_CODEC_ERROR		-8
#define GGRAPH_JPEG_CODEC_ERROR		-9
#define GGRAPH_TIFF_CODEC_ERROR		-10
#define GGRAPH_GEOTIFF_CODEC_ERROR	-11
#define GGRAPH_UNSUPPORTED_TIFF_LAYOUT	-12
#define GGRAPH_MISSING_GEO_DEFS		-13

/*
/ image access methods
*/
    GGRAPH_DECLARE void gGraphDestroyImage (const void *img);
    GGRAPH_DECLARE void gGraphDestroyImageInfos (const void *img);
    GGRAPH_DECLARE int gGraphGetImageDims (const void *img, int *width,
					   int *height);
    GGRAPH_DECLARE int gGraphGetImageInfos (const void *img, int *width,
					    int *height, int *colorspace,
					    int *max_palette,
					    int *bits_per_sample,
					    int *samples_per_pixel,
					    int *sample_format, int *tile_width,
					    int *tile_height,
					    int *rows_per_strip,
					    int *compression, int *scale_1_2,
					    int *scale_1_4, int *scale_1_8);
    GGRAPH_DECLARE int gGraphGetImageSize (const void *img, int *size);
    GGRAPH_DECLARE int gGraphImageColorSpaceOptimize (const void *img);
    GGRAPH_DECLARE int gGraphImageResampleAsMonochrome (const void *img);
    GGRAPH_DECLARE int gGraphImageResampleAsPalette (const void *img,
						     int num_colors);
    GGRAPH_DECLARE int gGraphImageResampleAsGrayscale (const void *img);
    GGRAPH_DECLARE int gGraphImageResampleAsPhotographic (const void *img);
    GGRAPH_DECLARE int gGraphImageGuessFormat (const void *mem_buf,
					       int mem_buf_size);
    GGRAPH_DECLARE int gGraphFileImageGuessFormat (const char *path, int *type);

/*
/ the following methods return a copy of the internal image buffer
/ PLEASE NOTE: you are responsible to free() the returned memory block
*/
    GGRAPH_DECLARE int gGraphImageBufferReferenceRGB (const void *img,
						      unsigned char **buffer);
    GGRAPH_DECLARE int gGraphImageBufferReferenceRGBA (const void *img,
						       unsigned char **buffer);
    GGRAPH_DECLARE int gGraphImageBufferReferenceARGB (const void *img,
						       unsigned char **buffer);
    GGRAPH_DECLARE int gGraphImageBufferReferenceBGR (const void *img,
						      unsigned char **buffer);
    GGRAPH_DECLARE int gGraphImageBufferReferenceBGRA (const void *img,
						       unsigned char **buffer);
    GGRAPH_DECLARE int gGraphImageBufferReferenceGDI (const void *img,
						      unsigned char **buffer);

/*
/ image resizing / image subsetting
*/
    GGRAPH_DECLARE int gGraphImageResizeNormal (const void *orig,
						const void **dest, int width,
						int height);
    GGRAPH_DECLARE int gGraphImageResizeHighQuality (const void *orig,
						     const void **dest,
						     int width, int height);
    GGRAPH_DECLARE int gGraphImageSubSet (const void *orig, const void **dest,
					  int upper_left_x, int upper_left_y,
					  int width, int height);

/*
/ utility functions handling image georeferencing
*/
    GGRAPH_DECLARE int gGraphGetWorldFilePath (const char *main_path,
					       char *world_file_path);
    GGRAPH_DECLARE int gGraphImageIsGeoReferenced (const void *img);
    GGRAPH_DECLARE int gGraphImageSetGeoReference (const void *img, int srid,
						   const char *srs_name,
						   const char *proj4text,
						   double upper_left_x,
						   double upper_left_y,
						   double pixel_x_size,
						   double pixel_y_size);
    GGRAPH_DECLARE int gGraphImageGetGeoReference (const void *img, int *srid,
						   char *srs_name,
						   char *proj4text,
						   double *upper_left_x,
						   double *upper_left_y,
						   double *pixel_x_size,
						   double *pixel_y_size);
    GGRAPH_DECLARE int gGraphImageInfosGetGeoReference (const void *img,
							int *srid,
							char *srs_name,
							char *proj4text,
							double *upper_left_x,
							double *upper_left_y,
							double *pixel_x_size,
							double *pixel_y_size);
    GGRAPH_DECLARE int gGraphReadWorldFile (const char *path,
					    double *upper_left_x,
					    double *upper_left_y,
					    double *pixel_x_size,
					    double *pixel_y_size);

/*
/ utility functions returning an image
*/
    GGRAPH_DECLARE int gGraphImageFromFile (const char *path, int image_type,
					    const void **image_handle,
					    int scale);
    GGRAPH_DECLARE int gGraphImageFromMemBuf (const void *mem_buf,
					      int mem_buf_size, int image_type,
					      const void **image_handle,
					      int scale);

/*
/ utility functions returning image infos
*/
    GGRAPH_DECLARE int gGraphImageInfosFromFile (const char *path,
						 int image_type,
						 const void **infos_handle);
    GGRAPH_DECLARE int gGraphImageInfosFromMemBuf (const void *mem_buf,
						   int mem_buf_size,
						   int image_type,
						   const void **infos_handle);

/*
/ utility functions generating a file from an image
*/
    GGRAPH_DECLARE int gGraphImageToJpegFile (const void *img, const char *path,
					      int quality);
    GGRAPH_DECLARE int gGraphImageToPngFile (const void *img, const char *path,
					     int compression_level,
					     int quantization_factor,
					     int interlaced);
    GGRAPH_DECLARE int gGraphImageToGifFile (const void *img, const char *path);
    GGRAPH_DECLARE int gGraphImageToTiffFile (const void *img, const char *path,
					      int tiff_layout,
					      int tile_width,
					      int tile_height,
					      int rows_per_strip,
					      int compression);
    GGRAPH_DECLARE int gGraphImageToGeoTiffFile (const void *img,
						 const char *path,
						 int tiff_layout,
						 int tile_width,
						 int tile_height,
						 int rows_per_strip,
						 int compression);

/*
/ utility functions generating a memory buffer from an image
*/
    GGRAPH_DECLARE int gGraphImageToJpegMemBuf (const void *img, void **mem_buf,
						int *mem_buf_size,
						int jpeg_quality);
    GGRAPH_DECLARE int gGraphImageToPngMemBuf (const void *img, void **mem_buf,
					       int *mem_buf_size,
					       int compression_level,
					       int quantization_factor,
					       int interlaced);
    GGRAPH_DECLARE int gGraphImageToGifMemBuf (const void *img, void **mem_buf,
					       int *mem_buf_size);
    GGRAPH_DECLARE int gGraphImageToTiffMemBuf (const void *img, void **mem_buf,
						int *mem_buf_size,
						int tiff_layout, int tile_width,
						int tile_height,
						int rows_per_strip,
						int compression);
    GGRAPH_DECLARE int gGraphImageToGeoTiffMemBuf (const void *img,
						   void **mem_buf,
						   int *mem_buf_size,
						   int tiff_layout,
						   int tile_width,
						   int tile_height,
						   int rows_per_strip,
						   int compression);

/*
/ functions for RAW image handling
*/
    GGRAPH_DECLARE int gGraphImageToRawMemBuf (const void *img, void **mem_buf,
					       int *mem_buf_size);
    GGRAPH_DECLARE int gGraphImageFromRawMemBuf (const void *mem_buf,
						 int mem_buf_size,
						 const void **image_handle,
						 int scale);
    GGRAPH_DECLARE int gGraphIsRawImage (const void *mem_buf, int mem_buf_size);

#ifdef __cplusplus
}
#endif

#endif				/* _GGRAPH_H */
