/* 
/ GGRAPH_internals.h
/
/ internal declarations
/
/ version  1.0, 2010 July 20
/
/ Author: Sandro Furieri a.furieri@lqt.it
/
/ Copyright (C) 2010  Alessandro Furieri
/
/    This program is free software: you can redistribute it and/or modify
/    it under the terms of the GNU Lesser General Public License as published by
/    the Free Software Foundation, either version 3 of the License, or
/    (at your option) any later version.
/
/    This program is distributed in the hope that it will be useful,
/    but WITHOUT ANY WARRANTY; without even the implied warranty of
/    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/    GNU Lesser General Public License for more details.
/
/    You should have received a copy of the GNU Lesser General Public License
/    along with this program.  If not, see <http://www.gnu.org/licenses/>.
/
*/

#define GG_IMAGE_JPEG_RGB		100
#define GG_IMAGE_JPEG_BW		101
#define GG_IMAGE_TIFF_FAX4		102
#define GG_IMAGE_TIFF_PALETTE	103
#define GG_IMAGE_TIFF_GRAYSCALE	104
#define GG_IMAGE_TIFF_RGB		105
#define GG_IMAGE_PNG_PALETTE	106
#define GG_IMAGE_PNG_GRAYSCALE	107
#define GG_IMAGE_PNG_RGB		108
#define GG_IMAGE_GIF_PALETTE	109

#define GG_PIXEL_UNKNOWN	-200
#define GG_PIXEL_RGB		201
#define GG_PIXEL_RGBA		202
#define GG_PIXEL_ARGB		203
#define GG_PIXEL_BGR		204
#define GG_PIXEL_BGRA		205
#define GG_PIXEL_GDI		206
#define GG_PIXEL_GRAYSCALE	207
#define GG_PIXEL_PALETTE	208
#define GG_PIXEL_GRID		209

#define GGRAPH_TRUE	-1
#define GGRAPH_FALSE	-2

#define GG_TARGET_IS_MEMORY	2001
#define GG_TARGET_IS_FILE	2002

#define GG_IMAGE_MAGIC_SIGNATURE	65391
#define GG_IMAGE_INFOS_MAGIC_SIGNATURE	37183

typedef struct gaia_graphics_image_infos
{
/* a generic image INFOS */
    int signature;
    int width;
    int height;
    int pixel_format;
    int bits_per_sample;
    int samples_per_pixel;
    int sample_format;
    int max_palette;
    int is_transparent;
    int tile_width;
    int tile_height;
    int rows_per_strip;
    int compression;
    int scale_1_2;
    int scale_1_4;
    int scale_1_8;
    int is_georeferenced;
    int srid;
    char *srs_name;
    char *proj4text;
    double upper_left_x;
    double upper_left_y;
    double pixel_x_size;
    double pixel_y_size;
} gGraphImageInfos;

typedef gGraphImageInfos *gGraphImageInfosPtr;

typedef struct gaia_graphics_image
{
/* a generic image  */
    int signature;
    unsigned char *pixels;
    int width;
    int height;
    int bits_per_sample;
    int samples_per_pixel;
    int sample_format;
    int scanline_width;
    int pixel_size;
    int pixel_format;
    int max_palette;
    unsigned char palette_red[256];
    unsigned char palette_green[256];
    unsigned char palette_blue[256];
    int is_transparent;
    unsigned char transparent_red;
    unsigned char transparent_green;
    unsigned char transparent_blue;
    int tile_width;
    int tile_height;
    int rows_per_strip;
    int compression;
    int is_georeferenced;
    int srid;
    char *srs_name;
    char *proj4text;
    double upper_left_x;
    double upper_left_y;
    double pixel_x_size;
    double pixel_y_size;
} gGraphImage;

typedef gGraphImage *gGraphImagePtr;

extern gGraphImageInfosPtr gg_image_infos_create (int pixel_format, int width,
						  int height,
						  int bits_per_sample,
						  int samples_per_pixels,
						  int sample_format,
						  const char *srs_name,
						  const char *proj4text);
extern void gg_image_infos_destroy (gGraphImageInfosPtr img);
extern gGraphImagePtr gg_image_create (int pixel_format, int width, int height,
				       int bits_per_sample,
				       int samples_per_pixels,
				       int sample_format, const char *srs_name,
				       const char *proj4text);
extern void gg_image_destroy (gGraphImagePtr img);
extern void gg_image_fill (const gGraphImagePtr img, unsigned char r,
			   unsigned char g, unsigned char b,
			   unsigned char alpha);
extern unsigned char gg_match_palette (const gGraphImagePtr img,
				       unsigned char r, unsigned char g,
				       unsigned char b);
extern int gg_set_image_transparent_color (unsigned char r, unsigned char g,
					   unsigned char b);
extern int gg_is_image_monochrome (const gGraphImagePtr img);
extern int gg_is_image_monochrome_ready (const gGraphImagePtr img);
extern int gg_is_image_grayscale (const gGraphImagePtr img);
extern int gg_is_image_palette256 (const gGraphImagePtr img);

extern int gg_georeference (int srid, const char *srs_name,
			    const char *proj4text, double upper_left_x,
			    double upper_left_y, double lower_right_x,
			    double lower_right_y);

extern void gg_make_thumbnail (const gGraphImagePtr thumbnail,
			       const gGraphImagePtr image);
extern void gg_image_resize (const gGraphImagePtr dst,
			     const gGraphImagePtr src);
extern void gg_image_clone_georeferencing (const gGraphImagePtr dst,
					   const gGraphImagePtr src);
extern void gg_image_sub_set (const gGraphImagePtr dst,
			      const gGraphImagePtr src, int upper_left_x,
			      int upper_left_y);

extern int gg_image_to_raw (const gGraphImagePtr img, void **mem_buf,
			    int *mem_buf_size);
extern int gg_image_to_jpeg (const gGraphImagePtr img, void **mem_buf,
			     int *mem_buf_size, FILE * out, int dest_type,
			     int quality);
extern int gg_image_to_png (const gGraphImagePtr img, void **mem_buf,
			    int *mem_buf_size, FILE * out, int dest_type,
			    int compression_level, int quantization_factor,
			    int interlaced);
extern int gg_image_to_gif (const gGraphImagePtr img, void **mem_buf,
			    int *mem_buf_size, FILE * out, int dest_type);
extern int gg_image_to_tiff_fax_strip (const gGraphImagePtr img, void **mem_buf,
				       int *mem_buf_size, FILE * out,
				       int dest_type, int rows_per_strip,
				       int compression);
extern int gg_image_to_tiff_fax_tile (const gGraphImagePtr img, void **mem_buf,
				      int *mem_buf_size, FILE * out,
				      int dest_type, int tile_width,
				      int tile_height, int compression);
extern int gg_image_to_tiff_palette_strip (const gGraphImagePtr img,
					   void **mem_buf, int *mem_buf_size,
					   FILE * out, int dest_type,
					   int rows_per_strip, int compression);
extern int gg_image_to_tiff_palette_tile (const gGraphImagePtr img,
					  void **mem_buf, int *mem_buf_size,
					  FILE * out, int dest_type,
					  int tile_width, int tile_height,
					  int compression);
extern int gg_image_to_tiff_grayscale_strip (const gGraphImagePtr img,
					     void **mem_buf, int *mem_buf_size,
					     FILE * out, int dest_type,
					     int rows_per_strip,
					     int compression);
extern int gg_image_to_tiff_grayscale_tile (const gGraphImagePtr img,
					    void **mem_buf, int *mem_buf_size,
					    FILE * out, int dest_type,
					    int tile_width, int tile_height,
					    int compression);
extern int gg_image_to_tiff_rgb_strip (const gGraphImagePtr img, void **mem_buf,
				       int *mem_buf_size, FILE * out,
				       int dest_type, int rows_per_strip,
				       int compression);
extern int gg_image_to_tiff_rgb_tile (const gGraphImagePtr img, void **mem_buf,
				      int *mem_buf_size, FILE * out,
				      int dest_type, int tile_width,
				      int tile_height, int compression);
extern int gg_image_to_tiff_grid_strip (const gGraphImagePtr img,
					void **mem_buf, int *mem_buf_size,
					FILE * out, int dest_type,
					int rows_per_strip, int compression);
extern int gg_image_to_tiff_grid_tile (const gGraphImagePtr img, void **mem_buf,
				       int *mem_buf_size, FILE * out,
				       int dest_type, int tile_width,
				       int tile_height, int compression);
extern int gg_image_to_geo_tiff_fax_strip (const gGraphImagePtr img,
					   void **mem_buf, int *mem_buf_size,
					   FILE * out, int dest_type,
					   int rows_per_strip, int compression);
extern int gg_image_to_geo_tiff_fax_tile (const gGraphImagePtr img,
					  void **mem_buf, int *mem_buf_size,
					  FILE * out, int dest_type,
					  int tile_width, int tile_height,
					  int compression);
extern int gg_image_to_geo_tiff_palette_strip (const gGraphImagePtr img,
					       void **mem_buf,
					       int *mem_buf_size, FILE * out,
					       int dest_type,
					       int rows_per_strip,
					       int compression);
extern int gg_image_to_geo_tiff_palette_tile (const gGraphImagePtr img,
					      void **mem_buf, int *mem_buf_size,
					      FILE * out, int dest_type,
					      int tile_width, int tile_height,
					      int compression);
extern int gg_image_to_geo_tiff_grayscale_strip (const gGraphImagePtr img,
						 void **mem_buf,
						 int *mem_buf_size, FILE * out,
						 int dest_type,
						 int rows_per_strip,
						 int compression);
extern int gg_image_to_geo_tiff_grayscale_tile (const gGraphImagePtr img,
						void **mem_buf,
						int *mem_buf_size, FILE * out,
						int dest_type, int tile_width,
						int tile_height,
						int compression);
extern int gg_image_to_geo_tiff_rgb_strip (const gGraphImagePtr img,
					   void **mem_buf, int *mem_buf_size,
					   FILE * out, int dest_type,
					   int rows_per_strip, int compression);
extern int gg_image_to_geo_tiff_rgb_tile (const gGraphImagePtr img,
					  void **mem_buf, int *mem_buf_size,
					  FILE * out, int dest_type,
					  int tile_width, int tile_height,
					  int compression);
extern int gg_image_to_geo_tiff_grid_strip (const gGraphImagePtr img,
					    void **mem_buf, int *mem_buf_size,
					    FILE * out, int dest_type,
					    int rows_per_strip,
					    int compression);
extern int gg_image_to_geo_tiff_grid_tile (const gGraphImagePtr img,
					   void **mem_buf, int *mem_buf_size,
					   FILE * out, int dest_type,
					   int tile_width, int tile_height,
					   int compression);

extern int gg_convert_image_to_rgb (const gGraphImagePtr img);
extern int gg_convert_image_to_rgba (const gGraphImagePtr img);
extern int gg_convert_image_to_argb (const gGraphImagePtr img);
extern int gg_convert_image_to_bgr (const gGraphImagePtr img);
extern int gg_convert_image_to_bgra (const gGraphImagePtr img);
extern int gg_convert_image_to_gdi (const gGraphImagePtr img);
extern int gg_convert_image_to_grayscale (const gGraphImagePtr img);
extern int gg_convert_image_to_palette (const gGraphImagePtr img);
extern int gg_image_resample_as_palette (const gGraphImagePtr img,
					 int num_colors);
extern int gg_convert_image_to_monochrome (const gGraphImagePtr img);

extern int gg_image_from_raw (int size, const void *data,
			      gGraphImagePtr * image_handle, int scale);
extern int gg_image_from_jpeg (int size, const void *data, int source_type,
			       gGraphImagePtr * image_handle, int scale);
extern int gg_image_from_png (int size, const void *data, int source_type,
			      gGraphImagePtr * image_handle, int scale);
extern int gg_image_from_gif (int size, const void *data, int source_type,
			      gGraphImagePtr * image_handle);
extern int gg_image_from_tiff (int size, const void *data, int source_type,
			       gGraphImagePtr * image_handle);
extern int gg_image_from_geo_tiff (int size, const void *data, int source_type,
				   gGraphImagePtr * image_handle);

extern int gg_image_infos_from_jpeg (int size, const void *data,
				     int source_type,
				     gGraphImageInfosPtr * infos_handle);
extern int gg_image_infos_from_png (int size, const void *data, int source_type,
				    gGraphImageInfosPtr * infos_handle);
extern int gg_image_infos_from_gif (int size, const void *data, int source_type,
				    gGraphImageInfosPtr * infos_handle);
extern int gg_image_infos_from_tiff (int size, const void *data,
				     int source_type,
				     gGraphImageInfosPtr * infos_handle);
extern int gg_image_infos_from_geo_tiff (int size, const void *data,
					 int source_type,
					 gGraphImageInfosPtr * infos_handle);

/* 
/
/ DISCLAIMER:
/ all the following code merely represents an 'ad hoc' adaption
/ deriving from the original GD lib code [BSD-like licensed]
/
*/

#define XGD_CTX_FREE	300
#define XGD_CTX_DONT_FREE	400

typedef struct xgdIOCtx
{
    int (*getC) (struct xgdIOCtx *);
    int (*getBuf) (struct xgdIOCtx *, void *, int);
    void (*putC) (struct xgdIOCtx *, int);
    int (*putBuf) (struct xgdIOCtx *, const void *, int);
    int (*seek) (struct xgdIOCtx *, const int);
    long (*tell) (struct xgdIOCtx *);
    void (*xgd_free) (struct xgdIOCtx *);
}
xgdIOCtx;

typedef struct xgdIOCtx *xgdIOCtxPtr;

typedef struct dpStruct
{
    FILE *file;
    void *data;
    int logicalSize;
    int realSize;
    int dataGood;
    int pos;
    int freeOK;
}
dynamicPtr;

typedef struct dpIOCtx
{
    xgdIOCtx ctx;
    dynamicPtr *dp;
}
dpIOCtx;

typedef struct dpIOCtx *dpIOCtxPtr;

extern int overflow2 (int a, int b);
extern void *xgdDPExtractData (struct xgdIOCtx *ctx, int *size);
extern xgdIOCtx *xgdNewDynamicCtx (int initialSize, const void *data,
				   int mem_or_file);
extern xgdIOCtx *xgdNewDynamicCtxEx (int initialSize, const void *data,
				     int freeOKFlag, int mem_or_file);
extern int xgdPutBuf (const void *buf, int size, xgdIOCtx * ctx);
extern int xgdGetBuf (void *, int, xgdIOCtx *);
